/*
 * LED color setting activity
 *  for Wi-Fi view & select
 */
package jp.gr.java_conf.j_sota.wifiviewandselect;

import java.util.List;

import android.app.Dialog;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.graphics.Color;
import android.net.wifi.WifiConfiguration;
import android.net.wifi.WifiManager;
import android.os.Bundle;
import android.preference.Preference;
import android.preference.PreferenceActivity;
import android.preference.PreferenceManager;
import android.preference.PreferenceScreen;
import android.preference.Preference.OnPreferenceClickListener;

public class LedColorSettingActivity extends PreferenceActivity {

	private static final int COLOR_PICKER_DIALOG = 1;

	private LedColorList mLedColorList;
	private Intent mReturnIntent;
	private PreferenceScreen mPreferenceScreen;

	/**
	 * Create activity
	 * @param savedInstanceState
	 */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		// Receive LED color setting list from main setting activity
		Intent intent = getIntent();
		mLedColorList = intent.getParcelableExtra(getString(R.string.key_led_setting));

		// Make preference screen
		setPreferenceScreen(createPreference());

		// Create Intent for return data to main setting activity
		mReturnIntent = new Intent();
		mReturnIntent.putExtra(getString(R.string.key_led_setting), mLedColorList);
		setResult(RESULT_OK, mReturnIntent);
	}

	/**
	 * Make preference screen
	 * @return PreferenceScreen	Preference screen
	 */
	private PreferenceScreen createPreference() {

		// Create blank preference screen
		mPreferenceScreen = getPreferenceManager().createPreferenceScreen(this);
		setPreferenceScreen(mPreferenceScreen);

		// Clear all used flag.
		for (int i = 0; i < mLedColorList.size(); i++) {
			mLedColorList.setUsed(i, false);
		}

		// Get configured Wi-Fi information
		Context context = getApplicationContext();
		WifiManager wifiManager = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);
		List<WifiConfiguration> configuredNetworkList = wifiManager.getConfiguredNetworks();
		if(configuredNetworkList != null) {
			final int defaultColorList[] = { Color.BLUE, Color.CYAN, Color.GREEN, Color.MAGENTA, Color.RED, Color.YELLOW };
			int lecColors = mLedColorList.size();

			// Process all configured Wi-Fi data
			for(int i = 0; i < configuredNetworkList.size(); i++) {
				String ssid = configuredNetworkList.get(i).SSID.replace("\"", "");
				// Search SSID in the LedColorList.
				boolean found = false;
				for (int j = 0; j < lecColors; j++) {
					if (ssid.equals(mLedColorList.getSsid(j))) {
						// This SSID is configured. Turn used flag on.
						mLedColorList.setUsed(j, true);
						found = true;
						break;
					}
				}
				// If configured SSID is not in the LedColorList, add it with default color.
				if (!found) {
					int color = Color.DKGRAY;
					int index = configuredNetworkList.get(i).networkId;
					if ((index >= 0) && (index < defaultColorList.length)) {
						color = defaultColorList[index];
					}
					mLedColorList.add(ssid, color, true);
				}
			}

			// Remove unconfigured SSID from LedColorList.
			for (int i = 0; i < mLedColorList.size(); i++) {
				if (!mLedColorList.getUsed(i)) {
					mLedColorList.remove(i);
					i--;
				}
			}
		}

		// Add LED color setting data to preference screen
		for (int i = 0; i < mLedColorList.size(); i++) {
			LedColorPreference ledColorPreference = new LedColorPreference(this);
			ledColorPreference.setKey(getString(R.string.key_led_color_preference) + i);
			ledColorPreference.setTitle(mLedColorList.getSsid(i));
			ledColorPreference.setSummary(getString(R.string.menu_led_color_set_summary));
			ledColorPreference.setLedColor(mLedColorList.getLedColor(i));
			ledColorPreference.setItemIndex(i);
			mPreferenceScreen.addPreference(ledColorPreference);

			// Set preference click listener
			ledColorPreference.setOnPreferenceClickListener(new OnPreferenceClickListener() {

				public boolean onPreferenceClick(Preference preference) {
					// Show color picker dialog. This is not DialogFragment.
					LedColorPreference ledColorPreference = (LedColorPreference)preference;
					Bundle data = new Bundle();
					int index = ledColorPreference.getItemIndex();
					data.putInt("INDEX", index);
					showDialog(COLOR_PICKER_DIALOG, data);
					return true;
				}
			});
		}
		return mPreferenceScreen;
	}

	/**
	 * Create dialog
	 * This method will be called when dialog shows first time.
	 * @param id		Dialog ID
	 * @param userData	Bundle data which passed to showDialog()
	 * @return Dialog Dialog
	 */
	protected Dialog onCreateDialog(int id, Bundle userData) {
		switch (id) {
		case COLOR_PICKER_DIALOG:
			// Create color picker dialog and set colorChanged listener
			ColorPickerDialog dialog = new ColorPickerDialog(this, new ColorPickerDialog.OnColorChangedListener() {

				/**
				 * This method is called when OK button on color picker dialog is clicked
				 * @param color		Selected color
				 * @param data		Bundle data which passed to showDialog() 
				 */
				@Override
				public void colorChanged(int color, Bundle data) {
					// Save color to LED color setting list
					int index = data.getInt("INDEX");
					mLedColorList.setLedColor(index, color);

					// Set LED color to LED color setting preference screen.
					LedColorPreference ledColorPreference = (LedColorPreference) mPreferenceScreen.getPreference(index);
					ledColorPreference.setLedColor(color);	// Update screen here
				}
			}, 0, userData);
			return dialog;

		default:
			return null;
		}
	}

	/**
	 * Prepare dialog
	 * This method will be called when every dialog show timing.
	 * @param id		Dialog ID
	 * @param dialog	Dialog
	 * @param userData	Bundle data which passed to showDialog()
	 */
	protected void onPrepareDialog(int id, Dialog dialog, Bundle userData) {
		switch (id) {
		case COLOR_PICKER_DIALOG:
			// Set default color and bundle data to color picker dialog
			ColorPickerDialog colorPicker = (ColorPickerDialog) dialog;
			colorPicker.prepare(mLedColorList.getLedColor(userData.getInt("INDEX")), userData);
			break;
		}
	}

	/**
	 * Back button pressed
	 */
	public void onBackPressed() {
		// Save LED color setting to default preference
		Context context = getApplicationContext();
		SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(context);
		Editor editor = pref.edit();
		int ledColors = mLedColorList.size();
		int used = 0;
		for (int i = 0; i < ledColors; i++) {
			if (mLedColorList.getUsed(i)) {
				String ssidKey = context.getString(R.string.key_led_setting_ssid) + used;
				String colorKey = context.getString(R.string.key_led_setting_color) + used;
				editor.putString(ssidKey, mLedColorList.getSsid(i));
				editor.putInt(colorKey, mLedColorList.getLedColor(i));
				used++;
			}
		}
		editor.putInt(context.getString(R.string.key_led_setting), used);
		editor.commit();
		super.onBackPressed();
	}
}
