/*
 * Main activity
 *  for Wi-Fi view & select
 */
package jp.gr.java_conf.j_sota.wifiviewandselect;

import java.lang.ref.WeakReference;
import java.util.List;

import android.net.wifi.ScanResult;
import android.net.wifi.WifiConfiguration;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.preference.PreferenceManager;
import android.app.Notification;
import android.app.NotificationManager;
import android.support.v4.app.NotificationCompat;
import android.app.PendingIntent;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.graphics.drawable.Drawable;
import android.support.v4.app.FragmentActivity;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.ToggleButton;

public class MainActivity extends FragmentActivity implements ConnectNotifyDialog.OnConnectButtonClickListener {

	private static final String TAG = "Wi-Fi V&S";
	private static final int PREFERENCE_MENU_CODE = 123;
	private static final int CONNECT_WAIT_TIMEOUT = 10*2;
	private static final int CONNECT_CHECK_WAIT = 500;
	private static final int WIFI_ONOFF_WAIT_TIMEOUT = 10*2;
	private static final int WIFI_ONOFF_CHECK_WAIT = 500;
	private static final int MAX_SIGNAL_LEVEL = 30;

	private volatile boolean mThreadAlive;
	private volatile boolean mThreadProc;
	private volatile int mInterval = 1;
	private boolean mLedOn = true;
	private boolean mNotifyOn = true;
	private LedColorList mLedColorList;
	private Thread mWifiScanThread;
	private Thread mWifiConnectThread = null;
	private Thread mWifiSwitchThread = null;
	private String mConnectingTo = null;
	private boolean mActive = false;
	private boolean mSwitchingToWifiOn = false;

	private Object lock = new Object();
	
	/**
	 * Create activity
	 * @param savedInstanceState
	 */
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);

		MyLog.d(TAG, "==== CREATE");

		loadApplicationSetting();
		initializeListView();
		initializeWifiOnOffButton();
		initializeConnectThread();
		initializeScanThread();
	}

	/**
	 * Load application setting
	 */
	private void loadApplicationSetting() {
		// Load from default preference
		SharedPreferences preference = PreferenceManager.getDefaultSharedPreferences(this);
		mInterval = Integer.parseInt(preference.getString(getString(R.string.key_interval), "1"));
		mLedOn = preference.getBoolean(getString(R.string.key_led_switch), true);
		mNotifyOn = preference.getBoolean(getString(R.string.key_notify_switch), true);
		int ledColors = preference.getInt(getString(R.string.key_led_setting), 0);
		mLedColorList = new LedColorList();
		for (int i = 0; i < ledColors; i++) {
			String ssidKey = getString(R.string.key_led_setting_ssid) + i;
			String colorKey = getString(R.string.key_led_setting_color) + i;
			String ssid = preference.getString(ssidKey, "");
			int color = preference.getInt(colorKey, 0);
			mLedColorList.add(ssid, color, true);
		}
	}

	/**
	 * Initialize list view
	 */
	private void initializeListView() {
		// Set empty adapter to ListView
		ConfiguredWifiList configuredWifi = new ConfiguredWifiList();
		ConfiguredWifiListAdapter adapter = new ConfiguredWifiListAdapter(this, configuredWifi.getArray(), mLedColorList);
		ListView configuredWifiView = (ListView) findViewById(R.id.listConfiguredWifi);
		configuredWifiView.setAdapter(adapter);
	}

	/**
	 * Initialize Wi-Fi ON/OFF button
	 */
	private void initializeWifiOnOffButton() {
		final ToggleButton wifiOnOffButton = (ToggleButton) findViewById(R.id.toggleButton1);
		synchronized(lock) {
			// Apply default state
			WifiManager wifiManager = (WifiManager) getSystemService(WIFI_SERVICE);
			if (wifiManager.isWifiEnabled()) {
				wifiOnOffButton.setChecked(true);
			} else {
				wifiOnOffButton.setChecked(false);
			}
		}

		final WifiOnOffThreadHandler handlerFromOnOffThread = new WifiOnOffThreadHandler(this);
		
		// Set Wi-Fi ON/OFF button change listener
		wifiOnOffButton.setOnCheckedChangeListener(new OnCheckedChangeListener() {

			/**
			 * This method will be called when Wi-Fi ON/OFF button is changed
			 * @param buttonView	Button
			 * @param checked		state
			 */
			@Override
			public void onCheckedChanged(CompoundButton buttonView, boolean checked) {
				final boolean wifiOn = checked;
				
				// Disable Wi-Fi ON/OFF button
				wifiOnOffButton.setEnabled(false);

				// Start Wi-Fi switch thread
				mWifiSwitchThread = new Thread() {
					@Override
					public void run() {
						switchWifiOnOffThread(wifiOn, handlerFromOnOffThread);
					}
				};
				mWifiSwitchThread.start();
			}
		});
	}

	/**
	 * Wi-Fi ON/OFF thread body
	 * @param wifiOn	ON/OFF switch (true=ON)
	 * @param handler	Handler of post thread processing
	 */
	private void switchWifiOnOffThread(boolean wifiOn, final Handler handler) {
		// Save state
		mSwitchingToWifiOn = wifiOn;

		// Update screen
		mWifiScanThread.interrupt();

		// Destroy Wi-Fi connect thread if alive
		if ((mWifiConnectThread != null) && mWifiConnectThread.isAlive()) {
			mWifiConnectThread.interrupt();
		}

		// Wi-Fi turn ON/OFF
		WifiManager wifiManager = (WifiManager) getSystemService(WIFI_SERVICE);
		synchronized(lock) {
			wifiManager.setWifiEnabled(wifiOn);
		}

		// Wait while ON/OFF is completed.
		boolean complete = false;
		boolean interrupted = false; 
		for (int i = 0; i < WIFI_ONOFF_WAIT_TIMEOUT; i++) {
			if (wifiOn && wifiManager.isWifiEnabled()) {
				complete = true;
				break;
			} else if (!wifiOn && !wifiManager.isWifiEnabled()) {
				complete = true;
				break;
			} else {
				try {
					Thread.sleep(WIFI_ONOFF_CHECK_WAIT);
				} catch (InterruptedException e) {
					interrupted = true;
					break;
				}
			}
		}
		
		// Send complete message to post processing
		Bundle data = new Bundle();
		data.putBoolean("COMPLETE", complete);
		data.putBoolean("INTERRUPTED", interrupted);

		Message message = handler.obtainMessage();
		message.setData(data);
		handler.sendMessage(message);
		MyLog.d(TAG, "===== WifiSwitchThread is terminated.");
	}

	/**
	 * Wi-Fi ON/OFF thread handler class.
	 * This is a static inner class. 
	 */
	static class WifiOnOffThreadHandler extends Handler {
		
		private final WeakReference<MainActivity> mActivity;

		/**
		 * Constructor.
		 * @param activity		activity
		 */
		WifiOnOffThreadHandler(MainActivity activity) {
			mActivity = new WeakReference<MainActivity>(activity);
		}

		/**
		 * Receive message from Wi-Fi ON/OFF thread.
		 * @param message		message
		 */
		@Override
		public void handleMessage(Message message) {
			MainActivity activity = mActivity.get();
			if (activity != null) {
				// Call post processing
				activity.procPostWifiOnOff(message);
			}
			return;
		}
	}

	/**
	 * Wi-Fi ON/OFF thread post processing
	 * @param message	Message from Wi-Fi ON/OFF thread
	 */
	private void procPostWifiOnOff(Message message) {
		Bundle data = message.getData();
		boolean complete = data.getBoolean("COMPLETE");
		boolean interrupted = data.getBoolean("INTERRUPTED");
		if (!complete && !interrupted) {
			// Turn on/off buton state
			ToggleButton wifiOnOffButton = (ToggleButton) findViewById(R.id.toggleButton1);
			wifiOnOffButton.setChecked(!wifiOnOffButton.isChecked());
			
			// Show Wi-Fi ON/OFF switch failed dialog or notification
			if (mActive) {
				// Show ON/OFF switch failed dialog
				SwitchFailedDialog switchFailedDialog = new SwitchFailedDialog();
				switchFailedDialog.show(getSupportFragmentManager(), "switchFailed");
			} else {
				// Show ON/OFF switch failed notification
				NotificationManager notificationManager = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
				Intent intent = new Intent(this, MainActivity.class);
				PendingIntent contentIntent = PendingIntent.getActivity(this, 0, intent, 0);
				Notification notification = new NotificationCompat.Builder(getApplicationContext())
					.setContentTitle(getString(R.string.app_name))
					.setContentText(getString(R.string.info_switch_failed))
					.setSmallIcon(R.drawable.ic_launcher)
					.setContentIntent(contentIntent)
					.setAutoCancel(true)
					.build();
				notificationManager.notify(0, notification);
			}
		}
		// Enable Wi-Fi ON/OFF button
		ToggleButton wifiOnOffButton = (ToggleButton) findViewById(R.id.toggleButton1);
		wifiOnOffButton.setEnabled(true);
	}

	/**
	 * Initialize connect thread
	 */
	private void initializeConnectThread() {
		// Set list view click listener
		ListView configuredWifiView = (ListView) findViewById(R.id.listConfiguredWifi);
		configuredWifiView.setOnItemClickListener(new AdapterView.OnItemClickListener() {

			/**
			 * This method will be called when list view item is clicked.
			 * @param parent	List view
			 * @param view		View
			 * @param position	Clicked position
			 * @param id		ID
			 */
			@Override
			public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
				// Get item from list view
				ListView listView = (ListView) parent;
				final ConfiguredWifiListData item = (ConfiguredWifiListData) listView.getItemAtPosition(position);
	
				if (mNotifyOn) {
					// Show connect notification dialog
					ConnectNotifyDialog connectNotifyDialog = new ConnectNotifyDialog();
					Bundle bundle = new Bundle();
					bundle.putString("SSID", item.ssid);
					bundle.putInt("NETWORKID", item.networkId);
					connectNotifyDialog.setArguments(bundle);
					connectNotifyDialog.show(getSupportFragmentManager(), "connectNotify");
				} else {
					// Start Wi-Fi connect
					startWifiConnect(item.networkId, item.ssid);
				}
			}
		});
	}

	/**
	 * ConnectNotifyDialog.OnConnectButtonClickListener
	 * This method will be called when connect button on connect notify dialog is clicked
	 * @param data		Bundle data which passed to ConnectNotifyDialog 
	 */
	@Override
	public void onConnectClicked(Bundle data) {
		// Start Wi-Fi connect
		startWifiConnect(data.getInt("NETWORKID"), data.getString("SSID"));
	}
	
	/**
	 * Start Wi-Fi connect
	 * @param networkId		Network ID
	 * @param ssid			SSID
	 */
	private void startWifiConnect(final int networkId, final String ssid) {
		// If connect thread is already alive, wait until thread is terminated
		if ((mWifiConnectThread != null) && mWifiConnectThread.isAlive()) {
			// Terminate connect thread
			mWifiConnectThread.interrupt();
			// Wait
			while (mWifiConnectThread.isAlive()) {
				MyLog.d(TAG, "##### Waiting...");
				try {
					Thread.sleep(500);
				} catch (InterruptedException e) {
				}
			}
		}

		// Show connecting to ...
		mConnectingTo = ssid;
		showConnecting(mConnectingTo);

		final ConnectThreadHandler handlerFromConnectThread = new ConnectThreadHandler(this);

		// start Wi-Fi connect thread
		mWifiConnectThread = new Thread() {
			@Override
			public void run() {
				wifiConnectThread(networkId, ssid, handlerFromConnectThread);
			}
		};
		mWifiConnectThread.start();
	}
	
	/**
	 * Wi-Fi connect thread body
	 * @param networkId		Network ID
	 * @param ssid			SSID
	 * @param handler		Handler of post thread processing
	 */
	private void wifiConnectThread(int networkId, String ssid, final Handler handler) {
		MyLog.d(TAG, "===== Try to connect " + networkId);
		boolean interrupted = false;
		boolean connected = false;

		WifiManager wifiManager = (WifiManager) getSystemService(WIFI_SERVICE);
		boolean useEnableNetworkMethod = false;
/*	  	try {
			// enableNetwork(id, false) does not work properly on Honeycomb.
			// Try to use connectNetwork() method instead.
			//    -> failure. comment out
			Method connectNetwork = wifiManager.getClass().getMethod("connectNetwork", int.class);
			try {
				connectNetwork.invoke(wifiManager,	networkId);
			} catch (IllegalArgumentException e) {
				MyLog.d(TAG, "===== IllegalArgumentException");
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				MyLog.d(TAG, "===== IllegalAccessException");
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				MyLog.d(TAG, "===== InvocationTargetException");
				e.printStackTrace();
			}
		} catch (NoSuchMethodException e1) */ {
			MyLog.d(TAG, "===== connectNetwork not found.");
			MyLog.d(TAG, "===== use enableNetwork.");
			useEnableNetworkMethod = true;
			synchronized(lock) {
				// Connect to Wi-Fi. Another configured Wi-Fi settings are disabled.
				wifiManager.enableNetwork(networkId, true);
			}
		}

		// Wait until connect complete
		MyLog.d(TAG, "===== Checking to connect " + networkId);
		for (int i = 0; i < CONNECT_WAIT_TIMEOUT; i++) {
			int connectedNetworkId;
			synchronized(lock) {
				WifiInfo wifiInfo = wifiManager.getConnectionInfo();
				connectedNetworkId = wifiInfo.getNetworkId();
			}
			if (connectedNetworkId == networkId) {
				connected = true;
				break;
			}
			try {
				Thread.sleep(CONNECT_CHECK_WAIT);
			} catch (InterruptedException e) {
				interrupted = true;
				break;
			}
		}

		if (connected) {
			MyLog.d(TAG, "===== Connected to " + networkId);
		}

		if (useEnableNetworkMethod && !interrupted) {
			if ((Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB) &&
					(Build.VERSION.SDK_INT <= Build.VERSION_CODES.HONEYCOMB_MR2)) {
				// do nothing
				// enableNetwork(id, false) causes thread hang up on Honeycomb.
				MyLog.d(TAG, "===== avoid enable all");
			} else {
				// Enable all configured Wi-Fi settings.
				MyLog.d(TAG, "===== enable all");
				synchronized(lock) {
					for (WifiConfiguration c0 : wifiManager.getConfiguredNetworks()) {
						MyLog.d(TAG, "===== enable " + c0.networkId);
						wifiManager.enableNetwork(c0.networkId, false);
						MyLog.d(TAG, "===== enable " + c0.networkId + " done");
					}
					MyLog.d(TAG, "===== enable all done");
					wifiManager.saveConfiguration();
				}
				MyLog.d(TAG, "===== save done");
			}
		}

		if (!interrupted) {
			// Send connect result to post processing
			Bundle data = new Bundle();
			data.putBoolean("CONNECTED", connected);
			data.putString("SSID", ssid);

			Message message = handler.obtainMessage();
			message.setData(data);
			handler.sendMessage(message);
			MyLog.d(TAG, "===== send message done");
		}
		MyLog.d(TAG, "===== WifiConnectThread is terminated.");
	}

	/**
	 * Wi-Fi connect thread handler class.
	 * This is a static inner class. 
	 */
	static class ConnectThreadHandler extends Handler {

		private final WeakReference<MainActivity> mActivity;

		/**
		 * Constructor
		 * @param activity		Activity
		 */
		ConnectThreadHandler(MainActivity activity) {
			mActivity = new WeakReference<MainActivity>(activity);
		}

		/**
		 * Receive message from connect thread
		 * @param message 	Message
		 */
		@Override
		public void handleMessage(Message message) {
			MainActivity activity = mActivity.get();
			if (activity != null) {
				// Call post processing
				activity.procPostWifiConnect(message);
			}
			return;
		}
	}

	/**
	 * Wi-Fi connect thread post processing
	 * @param message	Message from connect thread
	 */
	private void procPostWifiConnect(Message message) {
		// Update screen
		mWifiScanThread.interrupt();

		Bundle data = message.getData();
		boolean connected = data.getBoolean("CONNECTED", true);

		// Show dialog or notification if connect failed
		if (!connected) {
			String ssid = data.getString("SSID");
			if (mActive) {
				// Show connect failed dialog
				ConnectFailedDialog connectFailedDialog = new ConnectFailedDialog();
				Bundle bundle = new Bundle();
				bundle.putString("SSID", ssid);
				connectFailedDialog.setArguments(bundle);
				connectFailedDialog.show(getSupportFragmentManager(), "connectFailed");
			} else {
				// Show connect failed notification
				NotificationManager notificationManager = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
				Intent intent = new Intent(this, MainActivity.class);
				PendingIntent contentIntent = PendingIntent.getActivity(this, 0, intent, 0);
				Notification notification = new NotificationCompat.Builder(getApplicationContext())
					.setContentTitle(getString(R.string.app_name))
					.setContentText(getString(R.string.info_connect_failed_to_ssid, ssid))
					.setSmallIcon(R.drawable.ic_launcher)
					.setContentIntent(contentIntent)
					.setAutoCancel(true)
					.build();
				notificationManager.notify(0, notification);
			}
		}
	}

	/**
	 * Initialize Wi-Fi scan thread 
	 */
	private void initializeScanThread() {

		final ScanThreadHandler handlerFromScanThread = new ScanThreadHandler(this);
	
		// start Wi-Fi scan thread
		mThreadAlive = true;
		mWifiScanThread = new Thread() {
			@Override
			public void run() {
				wifiScanThread(handlerFromScanThread);
			}
		};
		mWifiScanThread.start();
	}

	/**
	 * Wi-Fi scan thread body
	 * @param handler Handler of show scan result
	 */
	private void wifiScanThread(Handler handler) {
		boolean interrupted = false;
		while (mThreadAlive) {
			if (mThreadProc || interrupted) {
				String ssid = getString(R.string.info_wifi_is_disabled);
				String information = "";
				int signalLevel = 0;
				int networkId = 0;
				ConfiguredWifiList configuredWifi = new ConfiguredWifiList();

				synchronized(lock) {
					WifiManager wifiManager = (WifiManager) getSystemService(WIFI_SERVICE);
					if (wifiManager != null) {
						if (wifiManager.isWifiEnabled()) {
							int wifiState = wifiManager.getWifiState();
							if (wifiState == WifiManager.WIFI_STATE_ENABLED) {
								// Get current connecting Wi-Fi information
								WifiInfo wifiInfo = wifiManager.getConnectionInfo();
								if ((wifiInfo.getNetworkId() >= 0) && (wifiInfo.getSSID() != null)) {
									ssid = wifiInfo.getSSID().replace("\"", "");
									networkId = wifiInfo.getNetworkId();
									int ipAddress = wifiInfo.getIpAddress();
									if (ipAddress == 0) {
										information = getString(R.string.info_getting_ip_address);
									} else {
										String ipAddressString = ((ipAddress >> 0) & 0xFF) + "." + ((ipAddress >> 8) & 0xFF) + "." + ((ipAddress >> 16) & 0xFF) + "." + ((ipAddress >> 24) & 0xFF);
										try {
											signalLevel = WifiManager.calculateSignalLevel(wifiInfo.getRssi(), MAX_SIGNAL_LEVEL) + 1;
										} catch (ArithmeticException e) {
											signalLevel = -1;
										}
										information = getString(R.string.info_information_format, wifiInfo.getLinkSpeed(), WifiInfo.LINK_SPEED_UNITS, ipAddressString);
									}
								} else {
									ssid = getString(R.string.info_not_connecting);
								}
							}
	
							// Get configured Wi-Fi information
							List<WifiConfiguration> configuredNetworkList = wifiManager.getConfiguredNetworks();
							if (configuredNetworkList != null) {
								for (int i = 0; i < configuredNetworkList.size(); i++) {
									String configuredSsid = configuredNetworkList.get(i).SSID.replace("\"", "");
									if (!configuredSsid.equals(ssid)) {
										// Add to configured Wi-Fi list
										int cfgNetworkId = configuredNetworkList.get(i).networkId;
										configuredWifi.add(configuredSsid, 0, cfgNetworkId);
									}
								}
	
								// Scan Wi-Fi and set signal level
								if (wifiManager.startScan()) {
									List<ScanResult> scanResultList = wifiManager.getScanResults();
									if (scanResultList != null) {
										for (int i = 0; i < configuredWifi.size(); i++) {
											for (int j = 0; j < scanResultList.size(); j++) {
												if (configuredWifi.getSsid(i).equals(scanResultList.get(j).SSID)) {
													try {
														configuredWifi.setSignalLevel(i, WifiManager.calculateSignalLevel(scanResultList.get(j).level, MAX_SIGNAL_LEVEL) + 1);
													} catch (ArithmeticException e) {
														configuredWifi.setSignalLevel(i, -1);
													}
													break;
												}
											}
										}
									}
								}
							}
						}
					}
				}

				// Send Scanned result to UI thread
				Bundle data = new Bundle();
				data.putString("SSID", ssid);
				data.putString("INFO", information);
				data.putInt("LEVEL", signalLevel);
				data.putInt("NETWORKID", networkId);
				data.putParcelable("NETS", configuredWifi);

				Message message = handler.obtainMessage();
				message.setData(data);
				handler.sendMessage(message);
				MyLog.d(TAG, "Update");

				// Reset interrupt flag
				if (interrupted) {
					interrupted = false;
					Thread.interrupted();
				}
			}

			try {
				// Wait
				Thread.sleep(calcWaitCount(mInterval));
			} catch (InterruptedException e) {
				MyLog.d(TAG, "Interrupted");
				interrupted = true;
			}
		}
		MyLog.d(TAG, "===== WiFi Scan thread is terminated");
	}

	/**
	 * Calculate wait count 
	 * @param interval		Interval (0...)
	 * @return int			wait count
	 */
	private int calcWaitCount(int interval) {
		int wait = 1000;
		final int waitList[] = { 500, 1000, 2000, 3000, 5000, 10000 };
		if ((interval >= 0) && (interval < waitList.length)) {
			wait = waitList[interval];
		}
		return wait;
	}

	/**
	 * Wi-Fi scan thread handler class.
	 * This is a static inner class. 
	 */
	static class ScanThreadHandler extends Handler {

		private final WeakReference<MainActivity> mActivity;

		/**
		 * Constructor
		 * @param activity	Activity
		 */
		ScanThreadHandler(MainActivity activity) {
			mActivity = new WeakReference<MainActivity>(activity);
		}

		/**
		 * Receive message from Wi-Fi scan thread
		 * @param message	Message from Wi-Fi scan thread
		 */
		@Override
		public void handleMessage(Message message) {
			MainActivity activity = mActivity.get();
			if (activity != null) {
				// Show Wi-Fi state
				activity.showWifiState(message);
			}
			return;
		}
	}

	/**
	 * Show Wi-Fi state
	 * @param message	Message from Wi-Fi scan thread
	 */
	private void showWifiState(Message message) {
		Bundle data = message.getData();

		if ((mWifiConnectThread != null) && mWifiConnectThread.isAlive()) {
			// Show connecting. (Now Wi-Fi connect thread is alive.)
			showConnecting(mConnectingTo);
		} else	if ((mWifiSwitchThread != null) && mWifiSwitchThread.isAlive()) {
			// Show switching. (Now Wi-Fi ON/OFF thread is alive.)
			showSwitching(mSwitchingToWifiOn);
		} else {
			// Show SSID of current connecting
			String ssid = data.getString("SSID");
			TextView mSsidView = (TextView) findViewById(R.id.textSsid);
			mSsidView.setText(ssid);

			// Show information of current connecting
			String info = data.getString("INFO");
			TextView mInfo1View = (TextView) findViewById(R.id.textInfo1);
			mInfo1View.setVisibility(View.VISIBLE);
			mInfo1View.setText(info);

			TextView mConnectTo1View = (TextView) findViewById(R.id.textConnectedTo);
			mConnectTo1View.setVisibility(View.INVISIBLE);

			// Show signal level of current connecting
			ImageView mLevelView = (ImageView) findViewById(R.id.imageSignalLevel);
			ImageView mLedView = (ImageView) findViewById(R.id.imageLed);
			if (info.length() == 0) {
				// If no informations
				mLevelView.setVisibility(View.INVISIBLE);
				mLedView.setVisibility(View.INVISIBLE);
			} else {
				mLevelView.setVisibility(View.VISIBLE);
				SignalLevelImage signalImage = new SignalLevelImage(this);
				Drawable dr = signalImage.getSignalImage(data.getInt("LEVEL"));
				mLevelView.setImageDrawable(dr);
				// Show LED of current connecting
				if (mLedOn) {
					mLedView.setVisibility(View.VISIBLE);
					LedImage ledImage = new LedImage(this);
					Bitmap bmp = ledImage.getLedImage(mLedColorList.getLedColorFromSsid(ssid, data.getInt("NETWORKID"), this));
					mLedView.setImageBitmap(bmp);
				} else {
					mLedView.setVisibility(View.INVISIBLE);
				}
			}
		}

		// Show configured Wi-Fi list
		// Change data in adapter for list view
		ConfiguredWifiList configuredWifi = data.getParcelable("NETS");
		ListView configuredWifiView = (ListView) findViewById(R.id.listConfiguredWifi);
		ConfiguredWifiListAdapter adapter = (ConfiguredWifiListAdapter) configuredWifiView.getAdapter();
		if (mLedOn) {
			adapter.setData(configuredWifi.getArray(), mLedColorList);
		} else {
			adapter.setData(configuredWifi.getArray(), null);
		}
		// Update screen
		adapter.notifyDataSetChanged();
	}

	/**
	 * Show Connecting to ... message
	 * @param ssid		SSID
	 */
	private void showConnecting(String ssid) {
		TextView mSsidView = (TextView) findViewById(R.id.textSsid);
		mSsidView.setText(getString(R.string.info_connecting_to));

		TextView mInfo1View = (TextView) findViewById(R.id.textInfo1);
		mInfo1View.setVisibility(View.INVISIBLE);

		TextView mConnectTo1View = (TextView) findViewById(R.id.textConnectedTo);
		mConnectTo1View.setVisibility(View.VISIBLE);
		mConnectTo1View.setText(ssid);

		ImageView mLevelView = (ImageView) findViewById(R.id.imageSignalLevel);
		mLevelView.setVisibility(View.INVISIBLE);

		ImageView mLedView = (ImageView) findViewById(R.id.imageLed);
		mLedView.setVisibility(View.INVISIBLE);
	}

	/**
	 * Show Wi-Fi switching to ... message
	 * @param toOn		ON/OFF flag (true=turn to on)
	 */
	private void showSwitching(boolean toOn) {
		TextView mSsidView = (TextView) findViewById(R.id.textSsid);
		if (toOn) {
			mSsidView.setText(getString(R.string.info_switching_to_on));
		} else {
			mSsidView.setText(getString(R.string.info_switching_to_off));
		}

		TextView mInfo1View = (TextView) findViewById(R.id.textInfo1);
		mInfo1View.setVisibility(View.INVISIBLE);

		TextView mConnectTo1View = (TextView) findViewById(R.id.textConnectedTo);
		mConnectTo1View.setVisibility(View.INVISIBLE);

		ImageView mLevelView = (ImageView) findViewById(R.id.imageSignalLevel);
		mLevelView.setVisibility(View.INVISIBLE);

		ImageView mLedView = (ImageView) findViewById(R.id.imageLed);
		mLedView.setVisibility(View.INVISIBLE);
	}

	/**
	 * This method will be called when activity goes to active.
	 */
	@Override
	protected void onResume() {
		// Start Wi-Fi scan processing
		mThreadProc = true;
		// Set active flag
		mActive = true;
		super.onResume();
		MyLog.d(TAG, "==== RESUME");
	}

	/**
	 * This method will be called when activity goes to inactive.
	 */
	@Override
	protected void onPause() {
		// Stop Wi-Fi scan processing
		mThreadProc = false;
		// Set active flag to inactive
		mActive = false;
		super.onPause();
		MyLog.d(TAG, "==== PAUSE");
	}

	/**
	 * This method will be called when activity goes to destroy.
	 */
	@Override
	protected void onDestroy() {
		// Destroy Wi-Fi scan thread
		mThreadAlive = false;
		mWifiScanThread.interrupt();

		// Destroy Wi-Fi connect thread if alive
		if ((mWifiConnectThread != null) && mWifiConnectThread.isAlive()) {
			mWifiConnectThread.interrupt();
		}
		
		// Destroy Wi-Fi switch thread if alive
		if ((mWifiSwitchThread != null) && mWifiSwitchThread.isAlive()) {
			mWifiSwitchThread.interrupt();
		}
		super.onDestroy();
		MyLog.d(TAG, "==== DESTROY");
	}

	/**
	 * Create option menu
	 * @param menu		Menu
	 * @return boolean
	 */
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}

	/**
	 * This method will be called when option menu is clicked.
	 * @param item		Menu item
	 * @return boolean 
	 */
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case  R.id.menu_settings:
			// Start setting activity
			Intent intent = new Intent(this, (Class<?>)MainSettingActivity.class);
			intent.putExtra(getString(R.string.key_led_setting), mLedColorList);
			startActivityForResult(intent, PREFERENCE_MENU_CODE);
			return true;

		case R.id.menu_about:
			// Show about dialog
			AboutDialog aboutDialog = new AboutDialog();
			aboutDialog.show(getSupportFragmentManager(), "about");
			return true;
		}
		return false;
	}

	/**
	 * Return from main setting activity
	 * @param requestCode	Request code
	 * @param resultCode	Result code
	 * @param data			Data from main setting activity
	 */
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		switch(requestCode) {
		case PREFERENCE_MENU_CODE:
			if (data != null) {
				// Receive settings from main setting activity
				mInterval = Integer.parseInt(data.getStringExtra(getString(R.string.key_interval)));
				mNotifyOn = data.getBooleanExtra(getString(R.string.key_notify_switch), true);
				mLedOn = data.getBooleanExtra(getString(R.string.key_led_switch), true);
				mLedColorList = data.getParcelableExtra(getString(R.string.key_led_setting));
	
				// Update screen
				mWifiScanThread.interrupt();
			}
			break;
		}
	}
}
